package com.dlodlo.otgdatainterface;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import com.dlodlo.dataprovider.IService;
import com.dlodlo.dataprovider.IServiceCallback;
import com.dlodlo.dataprovider.Quaternion;
import com.dlodlo.utils.CommonFuncs;
import com.dlodlo.utils.DataKey;
import com.dlodlo.utils.DloErrorCode;
import com.dlodlo.utils.DvrEventTag;
import com.dlodlo.utils.HandlerMsgTag;
import com.dlodlo.utils.MD5File;
import com.dlodlo.protocol.ThirdPartApp;
import com.dlodlo.utils.Utils;

import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.ServiceConnection;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.Signature;
import android.content.pm.PackageManager.NameNotFoundException;
import android.hardware.usb.UsbManager;
import android.net.Uri;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.util.Log;
import android.widget.Toast;

public class DataApi
{
	private static final int ALERT_DLG_INSTALL = 1;
	private static final int ALERT_DLG_USBATTACH = 2;
	private static final int ALERT_DLG_DISMISS = 3;
	
	private static final int BYTE_LENGTH = 1024;
	private byte[] mAllBytes = new byte[2*BYTE_LENGTH];
	private byte[] mSignatureBytes = new byte[BYTE_LENGTH];
	private int mSignatureBytesLength = 0;
	private byte[] mPackageBytes = new byte[BYTE_LENGTH];
	private int mPackageBytesLength = 0;
	
	private Context mContext = null;
	private IService otgService = null;
	
	private EventCallback  mEventCallback = null;
	private boolean isServiceBind = false;
	private boolean isBroadcastReceiver = false;
	private boolean isShowUsbAttachDlg = false;
	private AlertDialog alertDlg = null;
	private Handler mainThreadHandler = null;
	
	private String mBindPackageName = "";
	
	private IServiceCallback mCallback = new IServiceCallback.Stub() 
	{
		@Override
		public void pushKeyEvent(int keyState_, int keyCode_) throws RemoteException
		{			
			if(null != mEventCallback)
			{
				mEventCallback.pushKeyEvent(keyState_, keyCode_);
			}
		}

		@Override
		public void pushTouchEvent(int eventType_, int value1_, int value2_,
				int value3_) throws RemoteException
		{
			if(null != mEventCallback)
			{
				mEventCallback.pushTouchEvent(eventType_, value1_, value2_, value3_);
			}
		}

		@Override
		public void pushLogicSwitchEvent(int switchValue_)
				throws RemoteException
		{
			if(null != mEventCallback)
			{
				mEventCallback.pushLogicSwitchEvent(switchValue_);
			}
		}

		@Override
		public void pushProblem(int errorCode_) throws RemoteException 
		{
			if (errorCode_ == DloErrorCode.ERROR_USB_DISCONNECT)
			{
				return;
			}
			if(null != mEventCallback)
			{
				mEventCallback.pushProblem(errorCode_);
			}
		}

		@Override
		public void pushPermission(boolean hasPermission) throws RemoteException 
		{
			if(null != mEventCallback)
			{
				mEventCallback.pushPermission(hasPermission);
			}
		}

		@Override
		public void pushString(String string) throws RemoteException 
		{
			if(null != mEventCallback)
			{
				mEventCallback.pushString(string);
			}
		}
		
	};
	
	private void exitService()
	{
		if(null != otgService)
		{
			try 
			{
	            otgService.unregisterCallback(mCallback);
	        } 
			catch (RemoteException e) 
			{
	            e.printStackTrace();
	        }
		}
		
		otgService = null;
	}

	private ServiceConnection mConnection = new ServiceConnection() 
	{
		@Override
		public void onServiceDisconnected(ComponentName name) 
		{			
			release();
		}

		@Override
		public void onServiceConnected(ComponentName name, IBinder service) 
		{			
			otgService = IService.Stub.asInterface(service);
			if(null != otgService)
			{			
				try 
	            {				
					otgService.unregisterCallback(mCallback);
					otgService.registerCallback(mCallback);
					
					grantAuthorization(otgService);
					
					if(null != mEventCallback)
					{
						mEventCallback.pushSdkConnected(true);
					}
	            }
	            catch (RemoteException e) 
	            {
	                e.printStackTrace();
	            }
			}
			else
			{
				if(null != mEventCallback)
				{
					mEventCallback.pushSdkConnected(false);
				}
			}
		}
	};

	private BroadcastReceiver mUsbReceiver = new BroadcastReceiver() 
	{
		@Override
		public void onReceive(Context context, Intent intent) 
		{
			String action = intent.getAction();
			
			if(UsbManager.ACTION_USB_DEVICE_ATTACHED.equalsIgnoreCase(action)
				|| (UsbManager.ACTION_USB_ACCESSORY_ATTACHED.equalsIgnoreCase(action))
			)
			{
				if(otgService != null) 
				{
					try 
					{
						otgService.attachedReset();
					}
					catch (RemoteException e) 
					{
						e.printStackTrace();
						Utils.LOG("DataApi.java onReceive ACTION_USB_ACCESSORY_ATTACHED");
					}	
				}
			}
			else if (UsbManager.ACTION_USB_DEVICE_DETACHED.equalsIgnoreCase(action)
					|| UsbManager.ACTION_USB_ACCESSORY_DETACHED.equalsIgnoreCase(action))
			{
//				restartMloader();
				//showUsbAttachDialog();
			}
		}
	};
	
	public DataApi(Context context) 
	{
		init(context);
	}
	
	public DataApi(Context context, String packageName)
	{
		mBindPackageName = packageName;
		init(context);
	}
	
	public void init(Context context)
	{		
		mContext = context;
	}
	
	public void setEventCallback(EventCallback eventCallback_)
	{
		if(null != mEventCallback)
		{
			mEventCallback = null;
		}
		mEventCallback = eventCallback_;
	}
	
	public boolean hasPermission()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.hasPermission();
			}
			catch (RemoteException e) 
			{
				Log.e("darren", "", e);
				return false;
			}
		}
		
		return false;
	}
	
	public float[] getQuaternion()
	{
		if(null != otgService)
		{
			try
			{
				Quaternion quaternion = otgService.getQuaternion();
				
				if((null != quaternion) 
					&& (quaternion.isDeviceUsed())
				)
				{
					return quaternion.getValue();
				}
			}
			catch (RemoteException e) 
			{
				Log.e("darren", "", e);
				return new float[]{0.000000f, 0.000000f, 0.000000f, 0.000000f};
			}
		}
		
		return new float[]{0.000000f, 0.000000f, 0.000000f, 0.000000f};
	}
	
	public void setaID(String aID_)
	{
		if(null != otgService)
		{
			try 
			{
				otgService.setApplicationID(aID_);
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
			}
		}
	}
	
	public String getvS()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getvS();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	public String getvA()
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.getvA();
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	public String getaID()
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.getaID();
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	public String getpID()
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.getpID();
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	public String getSig()
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.getSig();
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	public boolean updateKey(String vA_, String kA_)
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.updateKey(vA_, kA_);
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return false;
			}
		}
		
		return false;
	}
	
	public boolean saveCfgFile(String aID_, String digest_)
	{
		if(null != otgService)
		{
			try 
			{
				return otgService.saveCfgFile(aID_, digest_);
			} 
			catch (RemoteException e) 
			{
				e.printStackTrace();
				return false;
			}
		}
		
		return false;
	}
	
	public String[] getAllAIDs()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getAllAIDs();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return null;
	}
	
	private boolean grantAuthorization(IService otgService)
	{		
		if(null != otgService)
		{
			try
			{				
				if(null != mContext)
				{
					String md5Str = getMd5String(mContext);

					if((null != md5Str)
						&& (!md5Str.isEmpty())
					)
					{								
						if(null != otgService)
						{									
							return otgService.permissionObtain(md5Str);
						}
					}
				}
				else
				{
					Log.d("darren", "dataApi mContext == null");
				}
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
		}
		
		return false;
	}
	
	public String getIcNumber()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getIcNumber();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return "unknown";
			}
		}
		
		return "unknown";
	}
	
	public int getPlatformSerial()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getPlatformSerial();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return -1;
			}
		}
		
		return -1;
	}
	
	public int getPlatformBatch()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getPlatformBatch();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return -1;
			}
		}
		
		return -1;
	}
	
	public int getSoftwareVersion()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getSoftwareVersion();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
				return -1;
			}
		}
		
		return -1;
	}
	
	public String getDeviceConfig()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getDeviceConfig();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
		}
		
		return "";
	}
	
	public String getCalibrationNumber()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.getData(DataKey.CALIBRATION_KEY);
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
		}
		
		return "";
	}
	
	public int getUsbConnectMode()
	{
		if(null != otgService)
		{
			try
			{
				String usbConnectMode = otgService.getData(DataKey.USB_CONNECT_MODE);
				if((null != usbConnectMode)
					&& (!usbConnectMode.isEmpty())
				)
				{
					int value = Integer.parseInt(usbConnectMode);
					return value;
				}
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
			catch(NumberFormatException e)
			{
				e.printStackTrace();
			}
		}
		
		return DvrEventTag.USB_NONE_CONNECTION;
	}
	
	public void setAuthInfo(byte[] authInfo_)
	{
		if(null != otgService)
		{
			try
			{
				otgService.setAuthInfo(authInfo_);
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
		}
	}
	
	public void transferData(String dataString)
	{
		if (null != otgService)
		{
			try
			{
				otgService.transferData(dataString);
			} catch (RemoteException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public boolean isUsbConnected()
	{
		if(null != otgService)
		{
			try
			{
				return otgService.isUsbConnected();
			}
			catch(RemoteException e)
			{
				e.printStackTrace();
			}
		}
		
		return false;
	}
	
	private String getMd5String(Context context_)
	{
		try
		{
			byte[] packageBytes = context_.getPackageName().getBytes("UTF-8");
			mPackageBytesLength = ((packageBytes.length >= BYTE_LENGTH) ? BYTE_LENGTH : packageBytes.length);
			System.arraycopy(packageBytes, 0, mPackageBytes, 0, mPackageBytesLength);
			
			PackageInfo packageInfo = context_.getPackageManager().getPackageInfo(context_.getPackageName(), PackageManager.GET_SIGNATURES);
			Signature[] signatures = packageInfo.signatures;
			mSignatureBytesLength = 0;
			int signatureArrayLength = signatures.length;
			int eachSignatureLength = BYTE_LENGTH/signatureArrayLength;
			for(int i=0; i<signatureArrayLength; ++i)
			{
				byte[] bytes = signatures[i].toByteArray();
				
				int realSignatureLength = ((bytes.length >= eachSignatureLength) ? eachSignatureLength : bytes.length);
				System.arraycopy(bytes, 0, mSignatureBytes, mSignatureBytesLength, realSignatureLength);
				mSignatureBytesLength += realSignatureLength;
			}
			
			System.arraycopy(mPackageBytes, 0, mAllBytes, 0, mPackageBytesLength);
			System.arraycopy(mSignatureBytes, 0, mAllBytes, mPackageBytesLength, mSignatureBytesLength);
			byte[] targetBytes = new byte[mPackageBytesLength+mSignatureBytesLength];
			System.arraycopy(mAllBytes, 0, targetBytes, 0, targetBytes.length);
			String targetMd5String = MD5File.getMD5String(targetBytes);
			return targetMd5String;
		}
		catch(NameNotFoundException e)
		{
			e.printStackTrace();
			return "";
		}
		catch(UnsupportedEncodingException e)
		{
			e.printStackTrace();
			return "";
		}
	}
	
	public void start() 
	{		
		synchronized(this)
		{			
			if(!isBroadcastReceiver)
			{
				isBroadcastReceiver = true;
				IntentFilter filter = new IntentFilter();
				filter.addAction(UsbManager.ACTION_USB_DEVICE_ATTACHED);
				filter.addAction(UsbManager.ACTION_USB_DEVICE_DETACHED);
				filter.addAction(UsbManager.ACTION_USB_ACCESSORY_ATTACHED);
				filter.addAction(UsbManager.ACTION_USB_ACCESSORY_DETACHED);
				filter.setPriority(Integer.MAX_VALUE);
				mContext.registerReceiver(mUsbReceiver, filter);
			}						
			
			if(!isServiceBind)
			{
				Intent levrService = new Intent(HandlerMsgTag.LEVR_SERVICE_ACTION);
				if(mBindPackageName.isEmpty())
				{
					levrService.setPackage(HandlerMsgTag.LEVR_SERVICE_PACKAGE_NAME);
				}
				else
				{
					levrService.setPackage(mBindPackageName);
				}
				
				isServiceBind = mContext.bindService(levrService, mConnection, Context.BIND_AUTO_CREATE);
			}
		}
	}
	
	public void stop()
	{
		exitService();
		
		if (null != mConnection && isServiceBind)
		{
			if(null != mEventCallback)
			{
				mEventCallback.pushSdkConnected(false);
			}
			
			mContext.unbindService(mConnection);
			isServiceBind = false;
		}
	}
	
	public void release()
	{
		stop();

		if (null != mUsbReceiver && isBroadcastReceiver) 
		{
			mContext.unregisterReceiver(mUsbReceiver);
			isBroadcastReceiver = false;
		}
	}
}
